import os
import urllib.request


def download_model(model_type):
    """
    model_type: str, A string representing the model type. It can be 'vit_h', 'vit_l', or 'vit_b'.
    """

    # A dictionary containing model types as keys and their respective URLs as values
    model_urls = {
        "vit_h": "https://dl.fbaipublicfiles.com/segment_anything/sam_vit_h_4b8939.pth",
        "vit_l": "https://dl.fbaipublicfiles.com/segment_anything/sam_vit_l_0b3195.pth",
        "vit_b": "https://dl.fbaipublicfiles.com/segment_anything/sam_vit_b_01ec64.pth",
    }

    # Check if the model file already exists and model_type is in model_urls
    filename = f"{model_type}.pth"
    if not os.path.exists(filename) and model_type in model_urls:
        url = model_urls[model_type]
        print(f"Downloading {model_type} model from {url}...")
        urllib.request.urlretrieve(url, filename)
        print(f"{model_type} model has been successfully downloaded and saved as '{filename}'.")
    elif os.path.exists(filename):
        print(f"{model_type} model already exists as '{filename}'. Skipping download.")
    else:
        raise ValueError("Invalid model type. It should be 'vit_h', 'vit_l', or 'vit_b'.")

    return filename



from typing import Any, Dict, List, Optional

import cv2
import numpy as np
import torch

from metaseg import SamAutomaticMaskGenerator, SamPredictor, sam_model_registry
from metaseg.utils import download_model, load_image, load_video


class SegAutoMaskPredictor:
    def __init__(self):
        self.model = None
        self.device = "cuda" if torch.cuda.is_available() else "cpu"

    def load_model(self, model_type):
        if self.model is None:
            self.model_path = download_model(model_type)
            self.model = sam_model_registry[model_type](checkpoint=self.model_path)
            self.model.to(device=self.device)

        return self.model

    def predict(self, frame, model_type, points_per_side, points_per_batch, min_area):
        model = self.load_model(model_type)
        mask_generator = SamAutomaticMaskGenerator(
            model, points_per_side=points_per_side, points_per_batch=points_per_batch, min_mask_region_area=min_area
        )

        masks = mask_generator.generate(frame)

        return frame, masks

    def save_image(self, source, model_type, points_per_side, points_per_batch, min_area):
        read_image = load_image(source)
        image, anns = self.predict(read_image, model_type, points_per_side, points_per_batch, min_area)
        if len(anns) == 0:
            return

        sorted_anns = sorted(anns, key=(lambda x: x["area"]), reverse=True)
        mask_image = np.zeros((anns[0]["segmentation"].shape[0], anns[0]["segmentation"].shape[1], 3), dtype=np.uint8)
        colors = np.random.randint(0, 255, size=(256, 3), dtype=np.uint8)
        for i, ann in enumerate(sorted_anns):
            m = ann["segmentation"]
            img = np.ones((m.shape[0], m.shape[1], 3), dtype=np.uint8)
            color = colors[i % 256]
            for i in range(3):
                img[:, :, 0] = color[0]
                img[:, :, 1] = color[1]
                img[:, :, 2] = color[2]
            img = cv2.bitwise_and(img, img, mask=m.astype(np.uint8))
            img = cv2.addWeighted(img, 0.35, np.zeros_like(img), 0.65, 0)
            mask_image = cv2.add(mask_image, img)

        combined_mask = cv2.add(image, mask_image)
        cv2.imwrite("output.jpg", combined_mask)

        return "output.jpg"

    def save_video(self, source, model_type, points_per_side, points_per_batch, min_area):
        cap, out = load_video(source)
        colors = np.random.randint(0, 255, size=(256, 3), dtype=np.uint8)

        while True:
            ret, frame = cap.read()
            if not ret:
                break

            image, anns = self.predict(frame, model_type, points_per_side, points_per_batch, min_area)
            if len(anns) == 0:
                continue

            sorted_anns = sorted(anns, key=(lambda x: x["area"]), reverse=True)
            mask_image = np.zeros(
                (anns[0]["segmentation"].shape[0], anns[0]["segmentation"].shape[1], 3), dtype=np.uint8
            )

            for i, ann in enumerate(sorted_anns):
                m = ann["segmentation"]
                color = colors[i % 256]  # Her nesne için farklı bir renk kullan
                img = np.zeros((m.shape[0], m.shape[1], 3), dtype=np.uint8)
                img[:, :, 0] = color[0]
                img[:, :, 1] = color[1]
                img[:, :, 2] = color[2]
                img = cv2.bitwise_and(img, img, mask=m.astype(np.uint8))
                img = cv2.addWeighted(img, 0.35, np.zeros_like(img), 0.65, 0)
                mask_image = cv2.add(mask_image, img)

            combined_mask = cv2.add(frame, mask_image)
            out.write(combined_mask)

        out.release()
        cap.release()
        cv2.destroyAllWindows()

        return "output.mp4"


class SegManualMaskPredictor:
    def __init__(self):
        self.model = None
        self.device = "cuda" if torch.cuda.is_available() else "cpu"

    def load_model(self, model_type):
        if self.model is None:
            self.model_path = download_model(model_type)
            self.model = sam_model_registry[model_type](checkpoint=self.model_path)
            self.model.to(device=self.device)

        return self.model

    def load_mask(self, mask, random_color):
        if random_color:
            color = np.random.rand(3) * 255
        else:
            color = np.array([100, 50, 0])
        h, w = mask.shape[-2:]
        mask_image = mask.reshape(h, w, 1) * color.reshape(1, 1, -1)
        mask_image = mask_image.astype(np.uint8)
        return mask_image

    def load_box(self, box, image):
        x0, y0 = box[0], box[1]
        x1, y1 = box[2], box[3]
        cv2.rectangle(image, (x0, y0), (x1, y1), (0, 255, 0), 2)
        return image

    def predict(
        self,
        frame,
        model_type,
        input_box=None,
        input_point=None,
        input_label=None,
        multimask_output=False,
    ):
        model = self.load_model(model_type)
        predictor = SamPredictor(model)

        predictor.set_image(frame)
        input_box = np.array(input_box)
        masks, _, _ = predictor.predict(
            point_coords=input_point,
            point_labels=input_label,
            box=input_box[None, :],
            multimask_output=multimask_output,
        )

        return frame, masks, input_box

    def save_image(
        self,
        source,
        model_type,
        input_box=None,
        input_point=None,
        input_label=None,
        multimask_output=False,
        output_path="output.jpg",
    ):
        read_image = load_image(source)
        image, anns, input_box = self.predict(
            read_image, model_type, input_box, input_point, input_label, multimask_output
        )
        if len(anns) == 0:
            return

        mask_image = self.load_mask(anns, True)
        image = self.load_box(input_box, image)
        combined_mask = cv2.add(image, mask_image)
        cv2.imwrite(output_path, combined_mask)

        return output_path



# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

from metaseg.modeling.image_encoder import ImageEncoderViT
from metaseg.modeling.mask_decoder import MaskDecoder
from metaseg.modeling.prompt_encoder import PromptEncoder
from metaseg.modeling.sam import Sam
from metaseg.modeling.transformer import TwoWayTransformer



# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

from metaseg.utils.data_utils import load_image, load_video
from metaseg.utils.file_utils import download_model



import cv2


def load_image(image_path):
    image = cv2.imread(image_path)
    image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
    return image


def load_video(video_path):
    cap = cv2.VideoCapture(video_path)
    frame_width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
    frame_height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
    fourcc = cv2.VideoWriter_fourcc(*"XVID")
    fps = int(cap.get(cv2.CAP_PROP_FPS))
    out = cv2.VideoWriter("output.mp4", fourcc, fps, (frame_width, frame_height))

    return cap, out
